import logging
import boto3

from spaceone.core.error import *
from spaceone.file_manager.connector.file_base_connector import FileBaseConnector

__all__ = ["AWSS3Connector"]
_LOGGER = logging.getLogger(__name__)


class AWSS3Connector(FileBaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.client = None
        self.bucket_name = None
        self._create_session()
        self._set_bucket()

    def _create_session(self):
        aws_access_key_id = self.config.get("aws_access_key_id")
        aws_secret_access_key = self.config.get("aws_secret_access_key")
        region_name = self.config.get("region_name")

        if region_name is None:
            raise ERROR_CONNECTOR_CONFIGURATION(backend="AWSS3Connector")

        if aws_access_key_id and aws_secret_access_key:
            self.client = boto3.client(
                "s3",
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                region_name=region_name,
            )
        else:
            self.client = boto3.client("s3", region_name=region_name)

    def _set_bucket(self):
        bucket_name = self.config.get("bucket_name")

        if bucket_name is None:
            raise ERROR_CONNECTOR_CONFIGURATION(backend="AWSS3Connector")

        self.bucket_name = bucket_name

    def get_upload_url(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        response = self.client.generate_presigned_post(self.bucket_name, object_name)
        return response["url"], response["fields"]

    def get_download_url(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        response = self.client.generate_presigned_url(
            "get_object", Params={"Bucket": self.bucket_name, "Key": object_name}
        )

        return response

    def check_file(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        try:
            self.client.get_object(Bucket=self.bucket_name, Key=object_name)
            return True
        except Exception as e:
            _LOGGER.debug(f"[check_file] get_object error: {e}")
            return False

    def delete_file(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        self.client.delete_object(Bucket=self.bucket_name, Key=object_name)

    @staticmethod
    def _generate_object_name(file_id, file_name):
        return f"{file_id}/{file_name}"



import logging
import functools
from typing import Union
from spaceone.api.file_manager.v1 import file_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.file_manager.model.file_model import File, FileReference

__all__ = ["FileInfo", "FilesInfo"]
_LOGGER = logging.getLogger(__name__)


def FileReferenceInfo(
    reference_vo: FileReference,
) -> Union[file_pb2.FileReference, None]:
    if reference_vo:
        info = {
            "resource_type": reference_vo.resource_type,
            "resource_id": reference_vo.resource_id,
        }

        return file_pb2.FileReference(**info)
    else:
        return None


def FileInfo(
    file_vo: File,
    minimal: bool = False,
    upload_url: str = None,
    upload_options: str = None,
    download_url: str = None,
) -> file_pb2.FileInfo:
    info = {
        "file_id": file_vo.file_id,
        "name": file_vo.name,
        "state": file_vo.state,
        "resource_group": file_vo.resource_group,
        "workspace_id": file_vo.workspace_id,
        "domain_id": file_vo.domain_id,
    }

    if minimal is False:
        info.update(
            {
                "file_type": file_vo.file_type,
                "tags": change_struct_type(file_vo.tags),
                "reference": FileReferenceInfo(file_vo.reference),
                "created_at": utils.datetime_to_iso8601(file_vo.created_at),
            }
        )

        if upload_url:
            info["upload_url"] = upload_url
            info["upload_options"] = (
                change_struct_type(upload_options) if upload_options else None
            )

        if download_url:
            info["download_url"] = download_url

    return file_pb2.FileInfo(**info)


def FilesInfo(file_vos, total_count, **kwargs):
    results = list(map(functools.partial(FileInfo, **kwargs), file_vos))

    return file_pb2.FilesInfo(results=results, total_count=total_count)



import logging
from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel

_LOGGER = logging.getLogger(__name__)


class FileReference(EmbeddedDocument):
    resource_type = StringField(required=True)
    resource_id = StringField(required=True)

    def to_dict(self):
        return dict(self.to_mongo())


class File(MongoModel):
    file_id = StringField(max_length=40, generate_id="file", unique=True)
    name = StringField(max_length=255, required=True)
    state = StringField(max_length=40, choices=("PENDING", "DONE"), default="PENDING")
    file_type = StringField(max_length=255, null=True, default=None)
    tags = DictField()
    reference = EmbeddedDocumentField(FileReference, null=True, default=None)
    resource_group = StringField(
        max_length=40, choices=("SYSTEM", "DOMAIN", "WORKSPACE", "PROJECT")
    )
    workspace_id = StringField(max_length=40, null=True, default=None)
    domain_id = StringField(max_length=40, null=True, default=None)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": ["state", "tags", "reference"],
        "minimal_fields": [
            "file_id",
            "name",
            "state",
            "reference",
            "resource_group",
            "workspace_id",
            "domain_id",
        ],
        "change_query_keys": {
            "resource_type": "reference.resource_type",
            "resource_id": "reference.resource_id",
        },
        "ordering": ["name"],
        "indexes": [
            "state",
            "file_type",
            "reference.resource_type",
            "reference.resource_id",
            "resource_group",
            "workspace_id",
            "domain_id",
        ],
    }



import logging

from spaceone.core.manager import BaseManager
from spaceone.file_manager.model.file_model import File

_LOGGER = logging.getLogger(__name__)


class FileManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.file_model: File = self.locator.get_model(File)

    def create_file(self, params: dict) -> File:
        def _rollback(vo: File) -> None:
            _LOGGER.info(f"[ROLLBACK] Delete file : {vo.name} ({vo.file_id})")
            vo.delete()

        file_vo: File = self.file_model.create(params)

        self.transaction.add_rollback(_rollback, file_vo)

        return file_vo

    def update_file_by_vo(self, params: dict, file_vo: File) -> File:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["file_id"]})'
            )
            file_vo.update(old_data)

        self.transaction.add_rollback(_rollback, file_vo.to_dict())

        return file_vo.update(params)

    @staticmethod
    def delete_file_by_vo(file_vo: File) -> None:
        file_vo.delete()

    def get_file(
        self,
        file_id: str,
        workspace_id: str = None,
        domain_id: str = None,
        project_id: str = None,
    ) -> File:
        conditions = {"file_id": file_id}

        if workspace_id:
            conditions.update({"workspace_id": workspace_id})

        if domain_id:
            conditions.update({"domain_id": domain_id})

        if project_id:
            conditions.update({"project_id": project_id})

        return self.file_model.get(**conditions)

    def list_files(self, query: dict) -> dict:
        return self.file_model.query(**query)

    def stat_files(self, query: dict) -> dict:
        return self.file_model.stat(**query)



import logging

from spaceone.core import config, cache
from spaceone.core.manager import BaseManager
from spaceone.file_manager.error import *
from spaceone.file_manager.connector.file_base_connector import FileBaseConnector

_LOGGER = logging.getLogger(__name__)


class FileConnectorManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        backend = config.get_global("BACKEND", "FileConnectorManager")
        try:
            _LOGGER.debug(f"[FileConnectorManager] Create {backend}")
            self.file_conn: FileBaseConnector = self.locator.get_connector(backend)
        except Exception as e:
            _LOGGER.error(f"[FileConnectorManager] not defined backend {backend}")
            raise ERROR_NOT_DEFINED_FILE_BACKEND(backend=backend)

    def get_upload_url(self, file_id, file_name):
        upload_url, upload_options = self.file_conn.get_upload_url(file_id, file_name)
        return upload_url, upload_options

    @cache.cacheable(
        key="file-manager:download-url:{domain_id}:{file_id}",
        expire=1800
    )
    def get_download_url(self, file_id: str, file_name: str, domain_id: str):
        download_url = self.file_conn.get_download_url(file_id, file_name)
        return download_url

    def check_file(self, file_id: str, file_name: str):
        return self.file_conn.check_file(file_id, file_name)

    def delete_file(self, file_id: str, file_name: str) -> None:
        self.file_conn.delete_file(file_id, file_name)



DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    "default": {
        "db": "file-manager",
        "host": "localhost",
        "port": 27017,
        "username": "",
        "password": "",
    }
}

CACHES = {
    "default": {},
    "local": {
        "backend": "spaceone.core.cache.local_cache.LocalCache",
        "max_size": 128,
        "ttl": 300,
    },
}

HANDLERS = {}

BACKEND = "AWSS3Connector"
CONNECTORS = {
    "AWSS3Connector": {
        "backend": "spaceone.file_manager.connector.aws_s3_connector:AWSS3Connector",
        "aws_access_key_id": "<optional>",
        "aws_secret_access_key": "<optional>",
        "region_name": "<required>",
        "bucket_name": "<required>",
    },
    "SpaceConnector": {
        "backend": "spaceone.core.connector.space_connector:SpaceConnector",
        "endpoints": {
            "identity": "grpc://identity:50051",
        },
    },
}

# System Token Settings
TOKEN = ""
